@@ -1,49 +1,21 @@
 /*
+ * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @license AGPL-3.0
  *
- *    OPEN-XCHANGE legal information
+ * This code is free software: you can redistribute it and/or modify
+ * it under the terms of the GNU Affero General Public License as published by
+ * the Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *    All intellectual property rights in the Software are protected by
- *    international copyright laws.
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Affero General Public License for more details.
  *
+ * You should have received a copy of the GNU Affero General Public License
+ * along with OX App Suite.  If not, see <https://www.gnu.org/licenses/agpl-3.0.txt>.
  *
- *    In some countries OX, OX Open-Xchange, open xchange and OXtender
- *    as well as the corresponding Logos OX Open-Xchange and OX are registered
- *    trademarks of the OX Software GmbH group of companies.
- *    The use of the Logos is not covered by the GNU General Public License.
- *    Instead, you are allowed to use these Logos according to the terms and
- *    conditions of the Creative Commons License, Version 2.5, Attribution,
- *    Non-commercial, ShareAlike, and the interpretation of the term
- *    Non-commercial applicable to the aforementioned license is published
- *    on the web site http://www.open-xchange.com/EN/legal/index.html.
- *
- *    Please make sure that third-party modules and libraries are used
- *    according to their respective licenses.
- *
- *    Any modifications to this package must retain all copyright notices
- *    of the original copyright holder(s) for the original code used.
- *
- *    After any such modifications, the original and derivative code shall remain
- *    under the copyright of the copyright holder(s) and/or original author(s)per
- *    the Attribution and Assignment Agreement that can be located at
- *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
- *    given Attribution for the derivative code and a license granting use.
- *
- *     Copyright (C) 2016-2020 OX Software GmbH
- *     Mail: info@open-xchange.com
- *
- *
- *     This program is free software; you can redistribute it and/or modify it
- *     under the terms of the GNU General Public License, Version 2 as published
- *     by the Free Software Foundation.
- *
- *     This program is distributed in the hope that it will be useful, but
- *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
- *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
- *     for more details.
- *
- *     You should have received a copy of the GNU General Public License along
- *     with this program; if not, write to the Free Software Foundation, Inc., 59
- *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Any use of the work other than as authorized under this license or copyright law is prohibited.
  *
  */
 
@@ -53,6 +25,7 @@ import static com.openexchange.authentication.LoginExceptionCodes.INVALID_CREDEN
 import static com.openexchange.authentication.LoginExceptionCodes.INVALID_CREDENTIALS_MISSING_USER_MAPPING;
 import static com.openexchange.authentication.LoginExceptionCodes.UNKNOWN;
 import static com.openexchange.java.Autoboxing.I;
+import java.io.IOException;
 import java.util.Optional;
 import java.util.Properties;
 import java.util.concurrent.TimeUnit;
@@ -70,6 +43,7 @@ import com.openexchange.authentication.AuthenticationService;
 import com.openexchange.authentication.LoginExceptionCodes;
 import com.openexchange.authentication.LoginInfo;
 import com.openexchange.config.ConfigurationService;
+import com.openexchange.config.ConfigurationServices;
 import com.openexchange.configuration.ConfigurationException;
 import com.openexchange.context.ContextService;
 import com.openexchange.exception.OXException;
@@ -101,7 +75,8 @@ public class IMAPAuthentication implements AuthenticationService {
         USE_MULTIPLE("USE_MULTIPLE"),
         IMAP_USE_SECURE("IMAP_USE_SECURE"),
         IMAPAUTHENC("com.openexchange.authentication.imap.imapAuthEnc"),
-        LOWERCASE_FOR_CONTEXT_USER_LOOKUP("LOWERCASE_FOR_CONTEXT_USER_LOOKUP");
+        LOWERCASE_FOR_CONTEXT_USER_LOOKUP("LOWERCASE_FOR_CONTEXT_USER_LOOKUP"),
+        AUTO_LOWER_CASE("com.openexchange.authentication.imap.autoLowerCase");
 
         /** The name of the property */
         public final String name;
@@ -114,6 +89,7 @@ public class IMAPAuthentication implements AuthenticationService {
     /**
      * The string for <code>ISO-8859-1</code> character encoding.
      */
+    @SuppressWarnings("unused")
     private static final String CHARENC_ISO8859 = "ISO-8859-1";
 
     // ----------------------------------------------------------------------------------------------------------------------
@@ -124,8 +100,10 @@ public class IMAPAuthentication implements AuthenticationService {
 
     /**
      * Default constructor.
+     * 
+     * @throws IOException if an I/O error is occurred
      */
-    public IMAPAuthentication(ServiceLookup services) {
+    public IMAPAuthentication(ServiceLookup services) throws IOException {
         super();
         this.services = services;
 
@@ -139,17 +117,17 @@ public class IMAPAuthentication implements AuthenticationService {
         }
 
         // Initialize configuration properties
-        props = configService.getFile("imapauth.properties");
+        props = ConfigurationServices.loadPropertiesFrom(configService.getFileByName("imapauth.properties"));
     }
 
     @Override
     public Authenticated handleLoginInfo(final LoginInfo loginInfo) throws OXException {
         try {
-            SplitResult splitResult = split(loginInfo.getUsername());
+            SplitResult splitResult = split(loginInfo.getUsername(), isAutoLowerCase());
 
             String localPart = splitResult.localPart;
             String password = loginInfo.getPassword();
-            if ("".equals(localPart.trim()) || "".equals(password.trim())) {
+            if (Strings.isEmpty(localPart) || Strings.isEmpty(password)) {
                 throw LoginExceptionCodes.INVALID_CREDENTIALS.create();
             }
 
@@ -241,34 +219,31 @@ public class IMAPAuthentication implements AuthenticationService {
                 Context ctx = optContext(contextInfo).get();
                 int userId = optUser(userInfo, loginInfo, ctx).get().intValue();
 
-	            // Load primary account and check its protocol to be IMAP
-	            MailAccount defaultMailAccount = optMailAccount(userId, ctx.getContextId()).get();
-	            String mailProtocol = defaultMailAccount.getMailProtocol();
-	            if (!mailProtocol.toLowerCase().startsWith("imap")) {
-	                throw UNKNOWN.create(new StringBuilder(128).append(
-	                    "IMAP authentication failed: Primary account's protocol is not IMAP but ").append(mailProtocol).append(
-	                    " for user ").append(userId).append(" in context ").append(ctx.getContextId()).toString());
-	            }
+                // Load primary account and check its protocol to be IMAP
+                MailAccount defaultMailAccount = optMailAccount(userId, ctx.getContextId()).get();
+                String mailProtocol = defaultMailAccount.getMailProtocol();
+                if (!mailProtocol.toLowerCase().startsWith("imap")) {
+                    throw UNKNOWN.create(new StringBuilder(128).append("IMAP authentication failed: Primary account's protocol is not IMAP but ").append(mailProtocol).append(" for user ").append(userId).append(" in context ").append(ctx.getContextId()).toString());
+                }
 
-	            /*
-	             * Set user according to configured login source if different from LoginSource.USER_NAME
-	             */
-	            final LoginSource loginSource = MailProperties.getInstance().getLoginSource(userId, ctx.getContextId());
-	            if (LoginSource.USER_IMAPLOGIN.equals(loginSource)) {
-	                imapLogin = defaultMailAccount.getLogin();
-	            }
-	            if (LoginSource.PRIMARY_EMAIL.equals(loginSource)) {
-	                imapLogin = defaultMailAccount.getPrimaryAddress();
-	            }
+                /*
+                 * Set user according to configured login source if different from LoginSource.USER_NAME
+                 */
+                final LoginSource loginSource = MailProperties.getInstance().getLoginSource(userId, ctx.getContextId());
+                if (LoginSource.USER_IMAPLOGIN.equals(loginSource)) {
+                    imapLogin = defaultMailAccount.getLogin();
+                } else if (LoginSource.PRIMARY_EMAIL.equals(loginSource)) {
+                    imapLogin = defaultMailAccount.getPrimaryAddress();
+                }
 
-	            /*
-	             * Get IMAP server from primary account
-	             */
-	            isPrimary = true;
-	            host = IDNA.toASCII(defaultMailAccount.getMailServer());
-	            port = Integer.valueOf(defaultMailAccount.getMailPort());
-	            secure = defaultMailAccount.isMailSecure();
-	            LOG.debug("Parsed IMAP Infos: {} {} {}  ({}@{})", (secure ? "imaps" : "imap"), host, port, Integer.valueOf(userId), Integer.valueOf(ctx.getContextId()));
+                /*
+                 * Get IMAP server from primary account
+                 */
+                isPrimary = true;
+                host = IDNA.toASCII(defaultMailAccount.getMailServer());
+                port = Integer.valueOf(defaultMailAccount.getMailPort());
+                secure = defaultMailAccount.isMailSecure();
+                LOG.debug("Parsed IMAP Infos: {} {} {}  ({}@{})", (secure ? "imaps" : "imap"), host, port, Integer.valueOf(userId), Integer.valueOf(ctx.getContextId()));
             } else {
                 // SSL feature for single defined IMAP server
                 if ("true".equalsIgnoreCase(props.getProperty(PropertyNames.IMAP_USE_SECURE.name))) {
@@ -423,6 +398,15 @@ public class IMAPAuthentication implements AuthenticationService {
         }
     }
 
+    private boolean isAutoLowerCase() {
+        boolean autoLowerCase = false;
+        Object value = props.get(PropertyNames.AUTO_LOWER_CASE.name);
+        if (value != null) {
+            autoLowerCase = Boolean.parseBoolean(value.toString().trim());
+        }
+        return autoLowerCase;
+    }
+
     private void applyEnableTls(Properties imapprops, ConfigurationService configuration) {
         String sEnableTls = configuration.getProperty("com.openexchange.imap.primary.enableTls", "").trim();
         if (Strings.isNotEmpty(sEnableTls)) {
@@ -497,17 +481,22 @@ public class IMAPAuthentication implements AuthenticationService {
      * Splits user name and context.
      *
      * @param loginInfo The composite information separated by an <code>'@'</code> sign
+     * @param autoLowerCase Whether to convert login info to lower-case
      * @return The split login info
      * @throws LoginException if no separator is found.
      */
-    private SplitResult split(String loginInfo) {
-        int pos = loginInfo.lastIndexOf('@');
+    private SplitResult split(String loginInfo, boolean autoLowerCase) {
+        String toSplit = loginInfo;
+        if (autoLowerCase) {
+            toSplit = Strings.asciiLowerCase(toSplit);
+        }
+        int pos = toSplit.lastIndexOf('@');
         if (pos <= 0) {
-            return new SplitResult(loginInfo, loginInfo, "defaultcontext");
+            return new SplitResult(toSplit, toSplit, "defaultcontext");
         }
 
         // Split by '@' character
-        return new SplitResult(loginInfo, loginInfo.substring(0, pos), loginInfo.substring(pos + 1));
+        return new SplitResult(toSplit, toSplit.substring(0, pos), toSplit.substring(pos + 1));
     }
 
     // ------------------------------------------------ Helper classes ------------------------------------------------
